#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Radar control program
import math
import gtk
import cairo
import pangocairo
import gobject
import serial

# PNG pictures loading class
class Image(object):

	def __init__(self, filename):
		self.set_file(filename)
		
	def set_file(self, filename):
		self.surface = cairo.ImageSurface.create_from_png(filename)
		
	def draw(self, cr):
		cr.save()
		cr.set_source_surface(self.surface)
		cr.paint()
		cr.restore()

# Function rotating against coordinates
def rotatexy(x, y, angle, cr):
	rad = math.radians(angle)
	s = math.sin(rad)
	c = math.cos(rad)
	matrix = cairo.Matrix(c, s, -s, c, x-c*x+s*y, y-s*x-c*y)
	cr.transform(matrix)

# Arduino communication class
class Hardware(object):

	def __init__(self, port):
		self.serial = serial.Serial(port, 9600)
		
	def set_direction(self, direction):
		self.serial.write('R%03i\n' % direction)
		
	def get_distance(self):
		# Double distance reading due to sensor errors
		self.serial.write('D')
		data = self.serial.readline()
		self.serial.write('D')
		data = self.serial.readline()
		return int(data[1:4])

# GTK Widget depicting the radar
class Radar(gtk.DrawingArea):
	
	def __init__(self):
		gtk.DrawingArea.__init__(self)
		self.distances = [0] * 180
		self.angle = 0
		self.direction = True #True == Right
		
		self.background = Image('background.png')
		self.left_hand = Image('left_hand.png')
		self.right_hand = Image('right_hand.png')
		self.glass = Image('glass.png')
		
		self.connect("expose-event", self.expose)
		self.set_size_request(512, 256)
		
	def draw(self, cr):
		cr.set_source_rgb(0, 0, 0)
		cr.paint()
		
		self.background.draw(cr)
		
		cr.save()
		rotatexy(255, 255, self.angle, cr)
		if self.direction:
			self.right_hand.draw(cr)
		else:
			self.left_hand.draw(cr)
		cr.restore()
		
		cr.save()
		cr.set_source_rgba(0, 1, 0, 0.2)
		cr.set_line_width(2)
		cr.move_to(0, 0)
		cr.line_to(0, 255)
		for angle in xrange(180):
			if self.distances[angle] > 0:
				rad = math.radians(angle + 180)
				x = math.cos(rad) * self.distances[angle] + 255
				y = math.sin(rad) * self.distances[angle] + 255
				cr.line_to(x, y)
		cr.line_to(511, 255)
		cr.line_to(511, 0)
		cr.close_path()
		cr.fill_preserve()
		cr.set_source_rgb(0, 1, 0)
		cr.stroke()
		cr.restore()
		
		cr.save()
		cr.translate(0, 175)
		self.glass.draw(cr)
		cr.restore()
		
	def expose(self, widget, event, data=None):
		cr = self.window.cairo_create()
		cr = pangocairo.CairoContext(cr)
		self.draw(cr)
		return False
		
	def refresh(self):
		alloc = self.get_allocation()
		rect = gtk.gdk.Rectangle(0, 0, alloc.width, alloc.height)
		self.window.invalidate_rect(rect, True)
		self.window.process_updates(True)
		
	def set_distance(self, angle, distance):
		self.angle = angle
		self.distances[self.angle] = distance
		self.refresh()

# Controller - application class
class App(gtk.Window):

	def __init__(self):
		gtk.Window.__init__(self)
		
		self.angle = 90
		self.direction = 10
		
		self.hardware = Hardware('/dev/ttyUSB0') # Set proper port here
		
		self.radar = Radar()
		self.add(self.radar)
		
		self.connect('destroy', self.close)
		
		self.show_all()
		
		self.set_resizable(False)
		
		gobject.timeout_add(500, self.rotate)
		
		gtk.main()
		
	def distance(self):
		if self.direction > 0:
			self.radar.direction = True
		else:
			self.radar.direction = False
		distance = self.hardware.get_distance()
		self.radar.set_distance(self.angle, distance)
		return False
		
	def rotate(self):
		self.angle += self.direction
		if self.angle > 179:
			self.angle = 179
			self.direction = -10
		elif self.angle < 0:
			self.angle = 0
			self.direction = 10
		
		self.hardware.set_direction(self.angle)
		gobject.timeout_add(50, self.distance)
		return True
		
	def close(self, widget, data=None):
		gtk.main_quit()


if __name__ == '__main__':
	App()


